disp(' ');	
disp(' envelope_fds.m   ver 1.2   March 13, 2014');
disp(' ');	
disp(' by Tom Irvine ');
disp(' Email:  tom@vibrationdata.com ');
disp(' ');	
disp(' This program determines an optimum envelope for an acceleration');
disp(' power spectral density.');
disp(' ');	
disp(' The optimum envelope is the that which yields the lowest RMS values. ');
disp(' ');	
disp(' The acceleration is assumed to be the base excitation for an array');
disp(' of independent, parallel single-degree-of-freedom systems.');
disp(' ');	
disp(' The Dirlik fatigue damage spectrum method is used to justify the'); 
disp(' envelope.');
disp(' ');	
disp(' The user supplied the amplification factor and fatigue exponent. ');
disp(' ');
disp(' The natural frequency is an independent variable.');
disp(' ');	
disp(' The user must input a time history file. ');
disp(' The file must be ASCII text with two columns: ');
disp(' Time(sec) Accel(G) ');
disp(' ');	
disp(' The format is free, but no header lines are allowed. ');
disp(' ');
%
clear t;
clear base;
clear input_fds;
clear best_psd;
clear best_fds;
clear fin;
clear psdint;
clear fn;
clear interp_psdin;
clear length;
clear damp;
clear bex;
clear fds_ref;
clear xfds;
%
close all;
%
fig_num=1;
%
tpi=2.*pi;
%
[t,base,dt,sr,tmx,tmi,n,ncontinue]=enter_time_history();
%
figure(fig_num);
fig_num=fig_num+1;
plot(t,base);
grid on;
title('Base Input');
ylabel('Accel(G)');
xlabel('Time(sec)');
%
ndc=input(' Enter number of damping cases (max=2) ');
if(ndc>2)
    ndc=2;
end    
%
Q=zeros(ndc,1);
damp=zeros(ndc,1);
for i=1:ndc
    out1=sprintf(' Enter Q value %d:  ',i);
    Q(i)=input(out1);
    damp(i)=1/(2*Q(i));
end    
%
disp(' ');
nfe=input(' Enter number of fatigue exponent cases (max=2) ');
if(nfe>2)
    nfe=2;
end    
%
bex=zeros(nfe,1);
for i=1:nfe
    out1=sprintf(' Enter fatigue exponent %d:  ',i);
    bex(i)=input(out1);
end
%
n_damp=length(damp);
n_bex=length(bex);
%
nm=max([ n_damp n_bex ]);
nb=n_damp*n_bex;  
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
T_out=input(' Enter output duration (sec)  ');
%
ncases=ndc*nfe;
%
MAX=20000;
MAX2=2000;
%
% nbreak   number of breakpoints
% ntrials  number of trails
% ntrials2
% n_ref    number of reference coordinates
%
record = 1.0e+90;
grmslow=1.0e+50;
vrmslow=1.0e+50;
drmslow=1.0e+50;
%
    disp(' ');
	disp(' Enter goal.  Minimize: ');
    disp(' ');
	disp(' 1=acceleration ');
	disp(' 2=acceleration, velocity ');
	disp(' 3=acceleration, velocty, displacement ');
	disp(' 4=acceleration, displacement ');
	disp(' 5=displacement ');
%
    goal=input(' ');	
%	
	disp(' Input number of trials ');
    ntrials=input(' ');
%
%
if(ntrials > MAX2)
		disp(' The maximum number of trials is 2000 ');
		ntrials=MAX2;
end
%
disp(' ');
disp(' Enter number of breakpoints for envelope (min=3 max=12): ');
nbreak=input(' ');
%
if(nbreak < 3  )
    nbreak=3;  
end
if(nbreak > 12 )
    nbreak=12; 
end
%
slope=zeros(nbreak,1);
fn=zeros(nbreak,1);
apsd_samfine=zeros(nbreak,1);
fds_samfine=zeros(nbreak,1);
xf=zeros(nbreak,1);
xapsd=zeros(nbreak,1);
%
disp(' ');
disp(' Constrain slopes?  1=yes 2=no ');
%   
ic=input(' '); 
%   
if(ic==1)
	   disp(' Enter absolute slope limit (dB/octave): ');
	   slopec=input(' ');
else
	   slopec = 60.;
end
%
slopec=abs(slopec);
slopec=(slopec/10.)/log10(2.);
%
disp(' ');
disp(' Further constrain initial slope to be positive ?  1=yes 2=no ');
initial=input(' ');
%   
disp(' ');
disp(' Further constrain final slope to be negative ?  1=yes 2=no ');
final=input(' ');
%
disp(' Input starting analysis frequency (Hz):  ');
f1=input(' ');
%   
disp(' Input ending analysis frequency (Hz):  ');
f2=input(' ');	
%
ffmin=f1;
ffmax=f2;
%
ocn=1./24.;
%
fn(1)=f1;
j=2;
while(1)
    fn(j)=fn(j-1)*2^(ocn);
    if(fn(j)>f2)
        fn(j)=f2;
        break;
    end
    j=j+1;
end
%
n_ref=length(fn);
% 
%%
[fds_ref]=env_fds_th(base,dt,fn,damp,bex);
%
for ik=1:ntrials
%	   
    if(rand()>0.5 || ik<20)
%	   
			% Generate the sample psd
            [f_sam,apsd_sam]=...
                env_generate_sample_psd(n_ref,nbreak,fn,ik,slopec,initial,final,f1,f2);            
%
    else
%
			[f_sam,apsd_sam]=...
                env_generate_sample_psd2(n_ref,nbreak,fn,xf,xapsd,slopec,initial,final,ik,f1,f2);
%
    end
%
%      Interpolate the sample psd
    [fn,apsd_samfine]=env_interp_sam(f_sam,apsd_sam,nbreak,n_ref,fn);
%                         
%      Calculate the fds of the sample psd
    [fds_samfine]=env_fds(apsd_samfine,n_ref,fn,damp,bex,T_out);    
%    
%      Compare the sample fds with the reference fds
    [scale]=env_compare(n_ref,fds_ref,fds_samfine,bex);
%
%      scale the psd
    scale=(scale^2.);
    apsd_sam=apsd_sam*scale;
%       
%      calculate the grms value 
%             
    [grms]=env_grms_sam(nbreak,f_sam,apsd_sam);
%
    [vrms]=env_vrms_sam(nbreak,f_sam,apsd_sam);
%
    [drms]=env_drms_sam(nbreak,f_sam,apsd_sam);
%	 
    [iflag,record]=env_checklow(grms,vrms,drms,grmslow,vrmslow,drmslow,record,goal);
%       
    if(iflag==1)
%           
        if(drms<drmslow)
            drmslow=drms;
        end
		if(vrms<vrmslow)
			vrmslow=vrms;
        end
		if(grms<grmslow)
			grmslow=grms;
        end
%           
		drmsp=drms;
		vrmsp=vrms;
		grmsp=grms;
%
        xf=f_sam;
 		xapsd=apsd_sam;	       
%
    end
%
    out1=sprintf('   Trial: drms=%10.4g  vrms=%10.4g  grms=%10.4g ',drms,vrms,grms); 
    out2=sprintf('  Record: drms=%10.4g  vrms=%10.4g  grms=%10.4g \n',drmsp,vrmsp,grmsp); 
    disp(out1);
    disp(out2);
%
end
%       
% Interpolate the best psd
%
sz=size(fn);
if(sz(2)>sz(1))
    fn=fn';
end 
%         
[xapsdfine]=env_interp_best(nbreak,n_ref,xf,xapsd,fn);
%
sz=size(xapsdfine);
if(sz(2)>sz(1))
    xapsdfine=xapsdfine';
end 
%
sz=size(xf);
if(sz(2)>sz(1))
    xf=xf';
end 
%
sz=size(xapsd);
if(sz(2)>sz(1))
    xapsd=xapsd';
end           
%
best_psd=[xf xapsd];
%
% Calculate the fds of the best psd
% 
[xfds]=env_fds(xapsdfine,n_ref,fn,damp,bex,T_out);
%
[grms]=env_grms_sam(nbreak,f_sam,xapsd);
%
[vrms]=env_vrms_sam(nbreak,f_sam,xapsd);
%
[drms]=env_drms_sam(nbreak,f_sam,xapsd);
%
disp('_____________________________________________________________________');
%
f_sam=xf;
apsd_sam=xapsd;  
power_spectral_density=[ f_sam   apsd_sam  ];
%
disp('Optimum Case');
disp(' ');
disp(' Freq(Hz)  Accel(G^2/Hz) ');
for i=1:nbreak
    out1=sprintf(' %6.1f \t%8.4g  ',f_sam(i),apsd_sam(i));
    disp(out1);
end    
%
out1=sprintf(' drms=%10.4g  vrms=%10.4g  grms=%10.4g ',drms,vrms,grms);
disp(out1);	
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
t_string=...
sprintf(' Power Spectral Density  Overall Level = %6.3g GRMS ',grms); 
ylab='Accel (G^2/Hz)';
x_label=sprintf(' Frequency (Hz)');
[fig_num]=plot_PSD(fig_num,x_label,ylab,t_string,power_spectral_density);
%
%
x_label=sprintf(' Frequency (Hz)');
ylab='Damage Index';
%
for i=1:length(damp)
    for j=1:length(bex)
%
        xx=zeros(n_ref,1);
        ff=zeros(n_ref,1);
%        
        for k=1:n_ref
            xx(k)=xfds(i,j,k);
            ff(k)=fds_ref(i,j,k);
        end
%
        xx=fix_size(xx);
        ff=fix_size(ff);
        fn=fix_size(fn);
%
        fds1=[fn xx];
        fds2=[fn ff];
%
        leg_a=sprintf('PSD Envelope');
        leg_b=sprintf('Measured Data');
%
         
%
        [fig_num]=...
        plot_fds_two(fig_num,x_label,ylab,fds1,fds2,leg_a,leg_b,Q(i),bex(j));
%
    end
end    
%
disp(' ');
disp(' The optimum PSD is stored in array:  power_spectral_density ');
disp(' ');
disp(' The fatigue damage spectra is calculated from the amplitude (peak-valley)/2 ');